@@ -7,22 +7,54 @@
 
 namespace mixal {
 
+Machine::Machine() : rA(), rX(), rI1(), rI2(), rI3(), rI4(), rI5(), rI6(), rJ(),
+      overflow(false), comparison(ComparisonIndicator::EQUAL), memory(),
+      devices(NUM_DEVICE, nullptr),
+      _pesudoVarIndex(), _lineOffset(), _elapsed(), _constants(), _lineNumbers() {}
+
 std::shared_ptr<IODevice> Machine::getDevice(int32_t index) {
     if (devices[index] == nullptr) {
         switch (index) {
         case 0: case 1: case 2: case 3:
         case 4: case 5: case 6: case 7:
-            devices[index] = std::unique_ptr<IODevice>(new IODeviceTape());
+            devices[index] = std::shared_ptr<IODevice>(new IODeviceTape());
             break;
         case 8: case 9: case 10: case 11:
         case 12: case 13: case 14: case 15:
-            devices[index] = std::unique_ptr<IODevice>(new IODeviceDisk());
+            devices[index] = std::shared_ptr<IODevice>(new IODeviceDisk());
+            break;
+        case 16:
+            devices[index] = std::shared_ptr<IODevice>(new IODeviceCardReader());
+            break;
+        case 17:
+            devices[index] = std::shared_ptr<IODevice>(new IODeviceCardPunch());
+            break;
+        case 18:
+            devices[index] = std::shared_ptr<IODevice>(new IODeviceLinePrinter());
+            break;
+        case 19:
+            devices[index] = std::shared_ptr<IODevice>(new IODeviceTypewriter());
+            break;
+        case 20:
+            devices[index] = std::shared_ptr<IODevice>(new IODevicePaperTape());
             break;
         }
     }
     return devices[index];
 }
 
+Register2& Machine::rI(int index) {
+    switch (index) {
+    case 1: return rI1;
+    case 2: return rI2;
+    case 3: return rI3;
+    case 4: return rI4;
+    case 5: return rI5;
+    case 6: return rI6;
+    }
+    throw RuntimeError(_lineOffset, "Invalid offset for index register: " + std::to_string(index));
+}
+
 const ComputerWord& Machine::memoryAt(int16_t index) const {
     return memory[index];
 }
@@ -34,8 +66,8 @@ ComputerWord& Machine::memoryAt(int16_t index) {
 void Machine::reset() {
     rA.reset();
     rX.reset();
-    for (int i = 0; i < NUM_INDEX_REGISTER; ++i) {
-        rI[i].reset();
+    for (int i = 1; i <= NUM_INDEX_REGISTER; ++i) {
+        rI(i).reset();
     }
     rJ.reset();
     overflow = false;
@@ -321,21 +353,24 @@ void Machine::executeSinglePesudo(ParsedResult* instruction) {
     }
 }
 
-void Machine::loadCodes(const std::vector<std::string>& codes) {
+void Machine::loadCodes(const std::vector<std::string>& codes, bool addHalt) {
     this->reset();
     // Parse and save all the results and intermediate expressions
     std::vector<ParsedResult> results(codes.size());
+    std::unordered_map<std::string, AtomicValue> evaluated;
     std::unordered_map<std::string, Expression*> expressions;
-    std::vector<std::tuple<std::string, Expression, Expression>> constants;
-    std::string lineBase;
-    int32_t lineOffset = 0;
+    std::vector<std::tuple<std::string, Expression, Expression>> constants;  // (name, address, value)
+    std::string lineBase = getPesudoSymbolname();
+    evaluated[lineBase] = AtomicValue(0);
+    int32_t lineOffset = 0, endIndex = -1;
     for (size_t codeIndex = 0; codeIndex < codes.size(); ++codeIndex) {
         auto code = codes[codeIndex];
         auto& result = results[codeIndex];
         auto lineSymbol = getPesudoSymbolname();
         result = Parser::parseLine(code, lineSymbol, true);
         if (result.parsedType == ParsedType::PSEUDO) {
-            switch (result.word.operation() + Instructions::PSEUDO) {
+            int32_t operation = result.word.operation() + Instructions::PSEUDO;
+            switch (operation) {
             case Instructions::EQU:
                 expressions[result.rawLocation] = &result.address;
                 break;
@@ -345,16 +380,16 @@ void Machine::loadCodes(const std::vector<std::string>& codes) {
                 lineOffset = 0;
                 break;
             case Instructions::CON:
-                if (lineBase.empty()) {
-                    constants.push_back({lineSymbol,
-                                         Expression::getConstExpression(AtomicValue(lineOffset)),
-                                         result.address});
-                } else {
-                    constants.push_back({lineSymbol,
-                                         Expression::getConstOffsetExpression(lineBase, lineOffset),
-                                         result.address});
+            case Instructions::ALF:
+                if (!result.rawLocation.empty()) {
+                    lineSymbol = result.rawLocation;
                 }
-                ++lineOffset;
+                constants.push_back({lineSymbol,
+                                     Expression::getConstOffsetExpression(lineBase, lineOffset++),
+                                     result.address});
+                break;
+            case Instructions::END:
+                endIndex = codeIndex;
                 break;
             default:
                 break;
@@ -364,11 +399,7 @@ void Machine::loadCodes(const std::vector<std::string>& codes) {
                                 (!result.rawAddress.empty() && result.address.depends().count(lineSymbol)) ||
                                 (!result.rawIndex.empty() && result.index.depends().count(lineSymbol)) ||
                                 (!result.rawField.empty() && result.field.depends().count(lineSymbol));
-            if (lineBase.empty()) {
-                result.location = Expression::getConstExpression(AtomicValue(lineOffset));
-            } else {
-                result.location = Expression::getConstOffsetExpression(lineBase, lineOffset);
-            }
+            result.location = Expression::getConstOffsetExpression(lineBase, lineOffset);
             if (!result.rawLocation.empty()) {
                 if (!(Atomic::isLocalSymbol(result.rawLocation) && result.rawLocation[1] == 'H')) {
                     expressions[result.rawLocation] = &result.location;
@@ -403,6 +434,14 @@ void Machine::loadCodes(const std::vector<std::string>& codes) {
         }
         result.address.replaceSymbol(localSymbolMapping);
     }
+    // Add halt
+    if (addHalt) {
+        auto lineSymbol = getPesudoSymbolname();
+        auto haltCommand = Parser::parseLine("HLT", lineSymbol, false);
+        constants.push_back({lineSymbol,
+                             Expression::getConstOffsetExpression(lineBase, lineOffset++),
+                             Expression::getConstExpression(AtomicValue(haltCommand.word.value()))});
+    }
     // Add expressions and literal constants
     for (auto& result : results) {
         if (!result.rawAddress.empty()) {
@@ -422,6 +461,12 @@ void Machine::loadCodes(const std::vector<std::string>& codes) {
             expressions[getPesudoSymbolname()] = &result.field;
         }
     }
+    if (endIndex != -1) {
+        auto lineSymbol = results[endIndex].rawLocation.empty() ? getPesudoSymbolname() : results[endIndex].rawLocation;
+        constants.push_back({lineSymbol,
+                             Expression::getConstOffsetExpression(lineBase, lineOffset++),
+                             results[endIndex].address});
+    }
     // Add collected constants to expressions
     for (auto& it : constants) {
         expressions[std::get<0>(it)] = &std::get<1>(it);
@@ -430,12 +475,14 @@ void Machine::loadCodes(const std::vector<std::string>& codes) {
     // Try to solve all the expressions
     std::unordered_map<std::string, int> dependNums;
     std::unordered_map<std::string, std::unordered_set<std::string>> solves;
-    std::unordered_map<std::string, AtomicValue> evaluated;
     std::set<std::pair<int, std::string>> tasks;
     for (auto& it : expressions) {
         dependNums[it.first] = static_cast<int>(it.second->depends().size());
         for (auto& depend : it.second->depends()) {
             solves[depend].insert(it.first);
+            if (evaluated.find(depend) != evaluated.end()) {
+                --dependNums[it.first];
+            }
         }
         tasks.insert({dependNums[it.first], it.first});
     }
@@ -472,7 +519,10 @@ void Machine::loadCodes(const std::vector<std::string>& codes) {
                                                        result.word.operation());
         }
     }
-    if (_lineOffset == -1) {
+    if (endIndex != -1) {
+        results[endIndex].address.evaluate(evaluated);
+        _lineOffset = results[endIndex].address.result().value;
+    } else if (_lineOffset == -1) {
         _lineOffset = 0;
     }
 }
@@ -484,7 +534,7 @@ std::string Machine::getPesudoSymbolname() {
 int Machine::getIndexedAddress(const InstructionWord& instruction) {
     int offset = 0;
     if (instruction.index() != 0) {
-        auto& rIi = rI[instruction.index() - 1];
+        auto& rIi = rI(instruction.index());
         offset = static_cast<int>(rIi.value());
     }
     return static_cast<int>(instruction.addressValue()) + offset;
