@@ -19,8 +19,8 @@ void Machine::executeCMP(const InstructionWord& instruction, Register5* reg) {
 }
 
 void Machine::executeCMPi(const InstructionWord& instruction) {
-    int registerIndex = instruction.operation() - Instructions::CMP1;
-    auto& rIi = rI[registerIndex];
+    int registerIndex = instruction.operation() - Instructions::CMP1 + 1;
+    auto& rIi = rI(registerIndex);
     ComputerWord t(rIi.sign, 0, 0, 0, rIi.byte1, rIi.byte2), a, b;
     int32_t address = getIndexedAddress(instruction);
     copyToRegister5(instruction, t, &a);
