@@ -18,18 +18,27 @@ void Machine::executeIOC(const InstructionWord& instruction) {
     if (device->type() == IODeviceType::DISK) {
         device->control(rX.value());
     } else {
-        device->control(instruction.addressValue());
+        int32_t address = getIndexedAddress(instruction);
+        device->control(address);
     }
 }
 
 void Machine::executeIN(const InstructionWord& instruction) {
     auto device = getDevice(instruction.field());
-    device->read(memory, instruction.addressValue());
+    if (!device->allowRead()) {
+        throw RuntimeError(_lineOffset, "Device does not support read: " + std::to_string(instruction.field()));
+    }
+    int32_t address = getIndexedAddress(instruction);
+    device->read(memory, address);
 }
 
 void Machine::executeOUT(const InstructionWord& instruction) {
     auto device = getDevice(instruction.field());
-    device->write(memory, instruction.addressValue());
+    if (!device->allowWrite()) {
+        throw RuntimeError(_lineOffset, "Device does not support write: " + std::to_string(instruction.field()));
+    }
+    int32_t address = getIndexedAddress(instruction);
+    device->write(memory, address);
 }
 
 void Machine::executeJRED(const InstructionWord& instruction) {
