@@ -10,8 +10,8 @@ void Machine::executeST(const InstructionWord& instruction, Register5* reg) {
 
 void Machine::executeSTi(const InstructionWord& instruction) {
     int address = getIndexedAddress(instruction);
-    int registerIndex = instruction.operation() - Instructions::ST1;
-    auto& rIi = rI[registerIndex];
+    int registerIndex = instruction.operation() - Instructions::ST1 + 1;
+    auto& rIi = rI(registerIndex);
     ComputerWord word(rIi.sign, 0, 0, 0, rIi[1], rIi[2]);
     copyFromRegister5(instruction, word, &memory[address]);
 }
